/*
 * @(#)SadissFileService.java 1.0 10/24/2002
 * Copyright 2002 Senamirmir Project
 *
 * You may distribute and/or modify Sadiss under the conditions 
 * of the LaTeX Project Public License. The license is located at
 * http://www.latex-project.org/lppl.txt
 */

package keffa ;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.filechooser.FileFilter ;
import javax.swing.text.* ;

/**
 * <!-- ----------------------------------------------------------------------- -->
 * SadissFileService is a class that is responsible for file related tasks 
 * including opening and saving.
 * @author abass alamene
 * @version 1.0 10/24/2002
 * <!-- ----------------------------------------------------------------------- -->
 */
class SadissFileService {
   private static SadissFileService sfs = new SadissFileService() ;
   private FileFilter ff ;
   private Component parent ;
   static final JFileChooser fc = new JFileChooser(".") ; //../sadiss0-7");

   /**
    * Empty Constructor
    */
   SadissFileService() {}

   /**
    * Empty Constructor
    * @param parent an instance of JFrame is used by Sadiss
    * @param ff filters for JFileChooser
    */
   SadissFileService(Component parent, FileFilter ff) {
      this.ff = ff ;
      this.parent = parent ;
   }

   /**
    * A static method that returns an instance of this class
    * @param parent an instance of JFrame is used by Sadiss
    * @param ff filters for JFileChooser
    */
   public static SadissFileService getInstance(Component parent, FileFilter ff)
      {
      sfs.ff = ff ;
      sfs.parent = parent ;
      return sfs ; 
      }

   /**
    * Returns a selected file 
    * @param owner a component that trigers this service
    * @param approveButtonText
    */
   public File saveGetFile(Component owner, String approveButtonText) {
      fc.addChoosableFileFilter(ff) ;
      int returnVal = fc.showDialog(parent, approveButtonText)  ;
      if (returnVal == JFileChooser.APPROVE_OPTION) {
         File file = fc.getSelectedFile();
         return file ;
      } else {
         System.out.println("User chose cancel") ;
         return null ;
      }
   }


   /**
    * Prompts a user with FileChooser to open file and returns a file object
    * @param approveButtonText type of JFileChooser
    */
   public File saveGetFile(String approveButtonText) {
      fc.addChoosableFileFilter(ff) ;
      int returnVal = fc.showDialog(parent, approveButtonText)  ;
      if (returnVal == JFileChooser.APPROVE_OPTION) {
         File file = fc.getSelectedFile();
         return file ;
      } else {
         System.out.println("User chose cancel") ;
         return null ;
      }
   }


   /**
    * Reads a file and returns the content of the file as StringBuffer object
    * @param file a file object from which the content will be read
    */
   public StringBuffer openSadissFile(File file)  {

      StringBuffer text   = new StringBuffer() ;
      String newLine      = "\n" ;
      String line         = "" ;

      try { 
         FileInputStream fis = new FileInputStream(file) ;
         
            // if the new file format is UTF8, read it as Unicode text
         String fileName = file.getName().toUpperCase() ;
         int dotIndex    = fileName.lastIndexOf('.') + 1 ;
         String ext      = fileName.substring( dotIndex, fileName.length() ) ;

         if ( ext.equals("UTF") | ext.equals("HTM") | ext.equals("HTML") | ext.equals("TEX")) {
             Reader r = new InputStreamReader(fis, "UTF8") ;
             BufferedReader br = new BufferedReader(r) ;
             while ( (line = br.readLine()) != null)
                text.append(line + newLine) ;

             br.close() ;
         } else if (ext.equals("UNI")) {
            BufferedInputStream bis = new BufferedInputStream(fis) ;
            DataInputStream dis = new DataInputStream(bis) ;
            try {
               while (true)
                  text.append(dis.readChar()) ;
            } catch (EOFException ex) {               
                  dis.close() ;
            }
         } else {
            Reader r = new InputStreamReader(fis, "ISO-8859-1") ;
            BufferedReader br = new BufferedReader(r) ;
            while ( (line = br.readLine()) != null)
               text.append(line + newLine) ;
            br.close() ;
         }
      } catch(Exception e) {
          System.out.println("Sadiss Eoror\n" + e) ;
      }
      return text ;
   }  // close method



   /**
    * Prompt the user for "Yes", "No", or "Cancel"
    * @param owner is an instance of SadissDocumentFrame
    * @param msg a text that should be displayed on the JOptionPane
    */
   public int isYesNoCancel(Component owner, String msg) {
      Object[] options = { "YES", "NO", "CANCEL" };

      int result = JOptionPane.showOptionDialog((SadissControl.SadissDocumentFrame) owner, 
                    msg, "Warning", JOptionPane.YES_NO_CANCEL_OPTION , 
                    JOptionPane.WARNING_MESSAGE, null, options, options[0]);

      return result ;         
   }

   /**
    * Prompts the user and returns 0 or 1
    * @param owner is an instance of SadissDocumentFrame
    * @param msg a text that should be displayed on the JOptionPane
    */
   public boolean isYesNo(Component owner, String msg) {
      Object[] options = { "NO", "YES" };
      int result = JOptionPane.showOptionDialog((SadissControl.SadissDocumentFrame) owner, 
                    msg, "Warning", JOptionPane.YES_NO_OPTION, 
                    JOptionPane.WARNING_MESSAGE, null, options, options[0]);

       switch(result) {
          case 0:
             return false ;
          case 1:
             return true ;
       }
       return false ; 
   }


   /**
    * Prompts the user whether to override existing file or not
    * @param owner is an instance of SadissDocumentFrame
    */
   public boolean isOverWriteAllowed(Component owner) {

       Object[] options = { "NO", "YES" };
       String msg   = "Overwrite existing file?" ;
       String title = "Warning" ;
       int jdo      = JOptionPane.DEFAULT_OPTION ; 
       int jwm      = JOptionPane.WARNING_MESSAGE ;

       int result = JOptionPane.showOptionDialog((SadissControl.SadissDocumentFrame) owner, 
                    msg, title, jdo, jwm, null, options, options[0]);

       switch(result) {
          case -1:
             return false ;
          case 0:
             return false ;
          case 1:
             return true ;
       }
       return false ; 
   }


   /**
    * Saves a document into a file
    * @param file the destination file where the document is saved
    * @param doc the document to be saved
    */
   public void saveSadissFile(File file, StyledDocument doc) {
      try {            
         FileOutputStream fos = new FileOutputStream(file) ;
         if ( file.getName().toUpperCase().endsWith(".UTF") | 
              file.getName().toUpperCase().endsWith(".HTM") |
              file.getName().toUpperCase().endsWith(".TEX") |              
              file.getName().toUpperCase().endsWith(".HTML") ) {

            Writer writer = new OutputStreamWriter(fos, "UTF-8") ;
            writer.write(doc.getText(0, doc.getLength()), 0, doc.getLength()) ;
//            writer.flush() ;
            writer.close() ;
         } else if ( file.getName().toUpperCase().endsWith(".UNI") ) {
            BufferedOutputStream bos = new BufferedOutputStream(fos) ;
  	         DataOutputStream dos = new DataOutputStream(bos) ;             
            dos.writeChars( doc.getText(0, doc.getLength()) ) ;
            dos.flush() ;
            dos.close() ;
         } else if ( file.getName().toUpperCase().endsWith(".ASCII") ) {
               // save the document as US ASCII by converting Unicode characters into
               // escape sequences
            Writer writer = new OutputStreamWriter(fos, "ISO-8859-1") ;
            BufferedWriter bw = new BufferedWriter(writer) ;
            String text = doc.getText(0, doc.getLength()) ;
   
            for (int i=0; i < text.length(); i++) {
               if (text.charAt(i) >= '\u1200') {
                  int code = (int) (text.charAt(i)) ;
                  bw.write( "\\u" + Integer.toHexString(code) ) ;
               } else {
                  bw.write(text.charAt(i)) ;
               }
            }
//            bw.flush() ;
            bw.close() ;
         } else {
            Writer writer = new OutputStreamWriter(fos, "ISO-8859-1") ;
            BufferedWriter bw = new BufferedWriter(writer) ;
            bw.write(doc.getText(0, doc.getLength()), 0, doc.getLength()) ;
            bw.flush() ;
            bw.close() ;
	      } 
      } catch(Exception e) {
         System.out.println(e) ;
      }
   }
}